@@ -1,11 +1,10 @@
 /*****************************************************************************
- *  Written by Chris Dunlap <cdunlap@llnl.gov>.
- *  Copyright (C) 2007-2022 Lawrence Livermore National Security, LLC.
+ *  Copyright (C) 2007-2025 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2002-2007 The Regents of the University of California.
  *  UCRL-CODE-155910.
  *
  *  This file is part of the MUNGE Uid 'N' Gid Emporium (MUNGE).
- *  For details, see <https://dun.github.io/munge/>.
+ *  For details, see <https://github.com/dun/munge>.
  *
  *  MUNGE is free software: you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
@@ -40,10 +39,10 @@
  */
 static const char *license_text[] = { \
     "Welcome to the MUNGE Uid 'N' Gid Emporium (MUNGE).",
-    "https://dun.github.io/munge/",
+    "https://github.com/dun/munge",
     "",
     "Written by Chris Dunlap <cdunlap@llnl.gov>.",
-    "Copyright (C) 2007-2022 Lawrence Livermore National Security, LLC.",
+    "Copyright (C) 2007-2025 Lawrence Livermore National Security, LLC.",
     "Copyright (C) 2002-2007 The Regents of the University of California.",
     "",
     "MUNGE is free software: you can redistribute it and/or modify it under",
