@@ -1,11 +1,10 @@
 /*****************************************************************************
- *  Written by Chris Dunlap <cdunlap@llnl.gov>.
- *  Copyright (C) 2007-2022 Lawrence Livermore National Security, LLC.
+ *  Copyright (C) 2007-2025 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2002-2007 The Regents of the University of California.
  *  UCRL-CODE-155910.
  *
  *  This file is part of the MUNGE Uid 'N' Gid Emporium (MUNGE).
- *  For details, see <https://dun.github.io/munge/>.
+ *  For details, see <https://github.com/dun/munge>.
  *
  *  MUNGE is free software: you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
@@ -31,11 +30,11 @@
 #endif /* HAVE_CONFIG_H */
 
 #include <assert.h>
-#include <sys/types.h>                  /* include before in.h for bsd */
 #include <netinet/in.h>
 #include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/types.h>
 #include <munge.h>
 #include "ctx.h"
 #include "munge_defs.h"
@@ -66,6 +65,7 @@ munge_ctx_create (void)
     ctx->socket_str = strdup (MUNGE_SOCKET_NAME);
     ctx->error_num = EMUNGE_SUCCESS;
     ctx->error_str = NULL;
+    ctx->flags = 0;
 
     if (!ctx->socket_str) {
         munge_ctx_destroy (ctx);
@@ -219,6 +219,14 @@ munge_ctx_get (munge_ctx_t ctx, int opt, ...)
             p2gid = va_arg (vargs, gid_t *);
             *p2gid = ctx->auth_gid;
             break;
+        case MUNGE_OPT_IGNORE_TTL:
+            p2int = va_arg (vargs, int *);
+            *p2int = !!(ctx->flags & MUNGE_CTX_FLAG_IGNORE_TTL);
+            break;
+        case MUNGE_OPT_IGNORE_REPLAY:
+            p2int = va_arg (vargs, int *);
+            *p2int = !!(ctx->flags & MUNGE_CTX_FLAG_IGNORE_REPLAY);
+            break;
         default:
             ctx->error_num = EMUNGE_BAD_ARG;
             break;
@@ -297,6 +305,18 @@ munge_ctx_set (munge_ctx_t ctx, int opt, ...)
         case MUNGE_OPT_GID_RESTRICTION:
             ctx->auth_gid = va_arg (vargs, gid_t);
             break;
+        case MUNGE_OPT_IGNORE_TTL:
+            if (va_arg (vargs, int))
+                ctx->flags |= MUNGE_CTX_FLAG_IGNORE_TTL;
+            else
+                ctx->flags &= ~MUNGE_CTX_FLAG_IGNORE_TTL;
+            break;
+        case MUNGE_OPT_IGNORE_REPLAY:
+            if (va_arg (vargs, int))
+                ctx->flags |= MUNGE_CTX_FLAG_IGNORE_REPLAY;
+            else
+                ctx->flags &= ~MUNGE_CTX_FLAG_IGNORE_REPLAY;
+            break;
         case MUNGE_OPT_ADDR4:
             /* this option cannot be set; fall through to error case */
         case MUNGE_OPT_ENCODE_TIME:
