@@ -1,11 +1,10 @@
 /*****************************************************************************
- *  Written by Chris Dunlap <cdunlap@llnl.gov>.
- *  Copyright (C) 2007-2016 Lawrence Livermore National Security, LLC.
+ *  Copyright (C) 2007-2025 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2002-2007 The Regents of the University of California.
  *  UCRL-CODE-155910.
  *
  *  This file is part of the MUNGE Uid 'N' Gid Emporium (MUNGE).
- *  For details, see <https://dun.github.io/munge/>.
+ *  For details, see <https://github.com/dun/munge>.
  *
  *  MUNGE is free software: you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
@@ -30,22 +29,25 @@
 #  include "config.h"
 #endif /* HAVE_CONFIG_H */
 
-#include <sys/types.h>                  /* include before grp.h for bsd */
 #include <assert.h>
 #include <ctype.h>
 #include <errno.h>
-#include <grp.h>
 #include <limits.h>
-#include <pwd.h>
+#include <signal.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/types.h>
 #include <unistd.h>
 #include <munge.h>
 #include "common.h"
 #include "license.h"
+#include "log.h"
+#include "munge_defs.h"
+#include "query.h"
 #include "read.h"
 #include "version.h"
+#include "xsignal.h"
 
 
 /*****************************************************************************
@@ -96,7 +98,7 @@ struct conf {
     int          dlen;                  /* payload data length               */
     void        *data;                  /* payload data                      */
     int          clen;                  /* munged credential length          */
-    char        *cred;                  /* munged credential nul-terminated  */
+    char        *cred;                  /* munged credential null-terminated */
 };
 
 typedef struct conf * conf_t;
@@ -111,8 +113,6 @@ void   destroy_conf (conf_t conf);
 void   parse_cmdline (conf_t conf, int argc, char **argv);
 void   display_help (char *prog);
 void   display_strings (const char *header, munge_enum_t type);
-uid_t  query_uid (const char *user);
-gid_t  query_gid (const char *group);
 void   open_files (conf_t conf);
 int    encode_cred (conf_t conf);
 void   display_cred (conf_t conf);
@@ -125,31 +125,23 @@ void   display_cred (conf_t conf);
 int
 main (int argc, char *argv[])
 {
-    conf_t       conf;
-    int          rc = 0;
-    const char  *p;
+    conf_t      conf;
+    const char *p;
 
-    if (posignal (SIGPIPE, SIG_IGN) == SIG_ERR) {
-        log_err (EMUNGE_SNAFU, LOG_ERR, "Failed to ignore signal=%d", SIGPIPE);
-    }
+    xsignal_ignore (SIGHUP);
+    xsignal_ignore (SIGPIPE);
     log_open_file (stderr, argv[0], LOG_INFO, LOG_OPT_PRIORITY);
     conf = create_conf ();
     parse_cmdline (conf, argc, argv);
     open_files (conf);
 
     if (conf->string) {
-        rc = read_data_from_string (conf->string, &conf->data, &conf->dlen);
+        conf->data = conf->string;
+        conf->dlen = strlen (conf->string);
     }
     else if (conf->fn_in) {
-        rc = read_data_from_file (conf->fp_in, &conf->data, &conf->dlen);
-    }
-    if (rc < 0) {
-        if (errno == ENOMEM) {
-            log_errno (EMUNGE_NO_MEMORY, LOG_ERR, "Failed to read input");
-        }
-        else {
-            log_err (EMUNGE_SNAFU, LOG_ERR, "Read error");
-        }
+        read_data_from_file (conf->fp_in, &conf->data, &conf->dlen,
+            MUNGE_MAXIMUM_PAYLOAD_LEN);
     }
     if (encode_cred (conf) < 0) {
         if (!(p = munge_ctx_strerror (conf->ctx))) {
@@ -162,6 +154,7 @@ main (int argc, char *argv[])
     display_cred (conf);
 
     destroy_conf (conf);
+    log_close_file ();
     exit (EMUNGE_SUCCESS);
 }
 
@@ -212,8 +205,10 @@ destroy_conf (conf_t conf)
         conf->fp_out = NULL;
     }
     if (conf->data != NULL) {
-        memburn (conf->data, 0, conf->dlen);
-        free (conf->data);
+        if (conf->data != conf->string) {
+            memburn (conf->data, 0, conf->dlen);
+            free (conf->data);
+        }
         conf->data = NULL;
     }
     if (conf->cred != NULL) {
@@ -328,10 +323,9 @@ parse_cmdline (conf_t conf, int argc, char **argv)
                 exit (EMUNGE_SUCCESS);
                 break;
             case 'u':
-                i = query_uid (optarg);
-                if (i < 0) {
+                if (query_uid (optarg, (uid_t *) &i) < 0) {
                     log_err (EMUNGE_SNAFU, LOG_ERR,
-                            "Unrecognized user \"%s\"", optarg);
+                        "Unrecognized user \"%s\"", optarg);
                 }
                 e = munge_ctx_set (conf->ctx, MUNGE_OPT_UID_RESTRICTION, i);
                 if (e != EMUNGE_SUCCESS) {
@@ -341,18 +335,16 @@ parse_cmdline (conf_t conf, int argc, char **argv)
                 }
                 break;
             case 'U':
-                i = query_uid (optarg);
-                if (i < 0) {
+                if (query_uid (optarg, (uid_t *) &i) < 0) {
                     log_err (EMUNGE_SNAFU, LOG_ERR,
-                            "Unrecognized user \"%s\"", optarg);
+                        "Unrecognized user \"%s\"", optarg);
                 }
-                conf->cuid = i;
+                conf->cuid = (uid_t) i;
                 break;
             case 'g':
-                i = query_gid (optarg);
-                if (i < 0) {
+                if (query_gid (optarg, (gid_t *) &i) < 0) {
                     log_err (EMUNGE_SNAFU, LOG_ERR,
-                            "Unrecognized group \"%s\"", optarg);
+                        "Unrecognized group \"%s\"", optarg);
                 }
                 e = munge_ctx_set (conf->ctx, MUNGE_OPT_GID_RESTRICTION, i);
                 if (e != EMUNGE_SUCCESS) {
@@ -362,26 +354,30 @@ parse_cmdline (conf_t conf, int argc, char **argv)
                 }
                 break;
             case 'G':
-                i = query_gid (optarg);
-                if (i < 0) {
+                if (query_gid (optarg, (gid_t *) &i) < 0) {
                     log_err (EMUNGE_SNAFU, LOG_ERR,
-                            "Unrecognized group \"%s\"", optarg);
+                        "Unrecognized group \"%s\"", optarg);
                 }
-                conf->cgid = i;
+                conf->cgid = (gid_t) i;
                 break;
             case 't':
                 errno = 0;
                 l = strtol (optarg, &p, 10);
-                if ((optarg == p) || (*p != '\0')) {
+                if ((optarg == p) || (*p != '\0') || (l < -1)) {
                     log_err (EMUNGE_SNAFU, LOG_ERR,
                         "Invalid time-to-live '%s'", optarg);
                 }
-                if (((errno == ERANGE) && (l == LONG_MAX)) || (l > INT_MAX)) {
+                if ((errno == ERANGE) && (l == LONG_MAX)) {
                     log_err (EMUNGE_SNAFU, LOG_ERR,
-                        "Exceeded maximum time-to-live of %d seconds",
-                        INT_MAX);
+                        "Overflowed maximum time-to-live of %ld seconds",
+                        LONG_MAX);
                 }
-                if (l < 0) {
+                if (l > UINT_MAX) {
+                    log_err (EMUNGE_SNAFU, LOG_ERR,
+                        "Exceeded maximum time-to-live of %u seconds",
+                        UINT_MAX);
+                }
+                if (l == -1) {
                     l = MUNGE_TTL_MAXIMUM;
                 }
                 e = munge_ctx_set (conf->ctx, MUNGE_OPT_TTL, (int) l);
@@ -463,7 +459,7 @@ display_help (char *prog)
     printf ("\n");
 
     printf ("  %*s %s\n", w, "-h, --help",
-            "Display this help");
+            "Display this help message");
 
     printf ("  %*s %s\n", w, "-L, --license",
             "Display license information");
@@ -476,30 +472,30 @@ display_help (char *prog)
     printf ("  %*s %s\n", w, "-n, --no-input",
             "Discard all input for payload");
 
-    printf ("  %*s %s\n", w, "-s, --string=STRING",
-            "Input payload from STRING");
+    printf ("  %*s %s\n", w, "-s, --string=STR",
+            "Input payload from string");
 
-    printf ("  %*s %s\n", w, "-i, --input=FILE",
-            "Input payload from FILE");
+    printf ("  %*s %s\n", w, "-i, --input=PATH",
+            "Input payload from file");
 
-    printf ("  %*s %s\n", w, "-o, --output=FILE",
-            "Output credential to FILE");
+    printf ("  %*s %s\n", w, "-o, --output=PATH",
+            "Output credential to file");
 
     printf ("\n");
 
-    printf ("  %*s %s\n", w, "-c, --cipher=STRING",
+    printf ("  %*s %s\n", w, "-c, --cipher=STR",
             "Specify cipher type");
 
     printf ("  %*s %s\n", w, "-C, --list-ciphers",
             "Display a list of supported ciphers");
 
-    printf ("  %*s %s\n", w, "-m, --mac=STRING",
+    printf ("  %*s %s\n", w, "-m, --mac=STR",
             "Specify MAC type");
 
     printf ("  %*s %s\n", w, "-M, --list-macs",
             "Display a list of supported MACs");
 
-    printf ("  %*s %s\n", w, "-z, --zip=STRING",
+    printf ("  %*s %s\n", w, "-z, --zip=STR",
             "Specify compression type");
 
     printf ("  %*s %s\n", w, "-Z, --list-zips",
@@ -508,22 +504,22 @@ display_help (char *prog)
     printf ("\n");
 
     printf ("  %*s %s\n", w, "-u, --restrict-uid=UID",
-            "Restrict credential decoding by UID");
+            "Restrict credential decoding by user/UID");
 
     printf ("  %*s %s\n", w, "-U, --uid=UID",
-            "Specify credential UID (requires root/CAP_SETUID)");
+            "Specify credential user/UID");
 
     printf ("  %*s %s\n", w, "-g, --restrict-gid=GID",
-            "Restrict credential decoding by GID");
+            "Restrict credential decoding by group/GID");
 
     printf ("  %*s %s\n", w, "-G, --gid=GID",
-            "Specify credential GID (requires root/CAP_SETGID)");
+            "Specify credential group/GID");
 
-    printf ("  %*s %s\n", w, "-t, --ttl=INTEGER",
+    printf ("  %*s %s\n", w, "-t, --ttl=SECS",
             "Specify time-to-live (in seconds; 0=dfl -1=max)");
 
-    printf ("  %*s %s\n", w, "-S, --socket=STRING",
-            "Specify local domain socket for munged");
+    printf ("  %*s %s\n", w, "-S, --socket=PATH",
+            "Specify local socket for munged");
 
     printf ("\n");
     printf ("By default, payload read from stdin, "
@@ -551,78 +547,6 @@ display_strings (const char *header, munge_enum_t type)
 }
 
 
-uid_t
-query_uid (const char *user)
-{
-/*  Returns the uid for [user], or -1 on error.
- */
-    struct passwd *pw_ptr;
-    uid_t          uid;
-    long int       luid;
-    char          *end_ptr;
-
-    if (user == NULL) {
-        errno = EINVAL;
-        return (-1);
-    }
-    pw_ptr = getpwnam (user);
-    if (pw_ptr != NULL) {
-        uid = pw_ptr->pw_uid;
-    }
-    else {
-        errno = 0;
-        luid = strtol (user, &end_ptr, 10);
-        if ((errno == ERANGE) && ((luid == LONG_MIN) || (luid == LONG_MAX))) {
-            return (-1);
-        }
-        if ((user == end_ptr) || (*end_ptr != '\0')) {
-            return (-1);
-        }
-        if ((luid < 0) || (luid > INT_MAX)) {
-            return (-1);
-        }
-        uid = (int) luid;
-    }
-    return (uid);
-}
-
-
-gid_t
-query_gid (const char *group)
-{
-/*  Returns the gid for [group], or -1 on error.
- */
-    struct group  *gr_ptr;
-    gid_t          gid;
-    long int       lgid;
-    char          *end_ptr;
-
-    if (group == NULL) {
-        errno = EINVAL;
-        return (-1);
-    }
-    gr_ptr = getgrnam (group);
-    if (gr_ptr != NULL) {
-        gid = gr_ptr->gr_gid;
-    }
-    else {
-        errno = 0;
-        lgid = strtol (group, &end_ptr, 10);
-        if ((errno == ERANGE) && ((lgid == LONG_MIN) || (lgid == LONG_MAX))) {
-            return (-1);
-        }
-        if ((group == end_ptr) || (*end_ptr != '\0')) {
-            return (-1);
-        }
-        if ((lgid < 0) || (lgid > INT_MAX)) {
-            return (-1);
-        }
-        gid = (int) lgid;
-    }
-    return (gid);
-}
-
-
 void
 open_files (conf_t conf)
 {
