@@ -1,11 +1,10 @@
 /*****************************************************************************
- *  Written by Chris Dunlap <cdunlap@llnl.gov>.
- *  Copyright (C) 2007-2016 Lawrence Livermore National Security, LLC.
+ *  Copyright (C) 2007-2025 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2002-2007 The Regents of the University of California.
  *  UCRL-CODE-155910.
  *
  *  This file is part of the MUNGE Uid 'N' Gid Emporium (MUNGE).
- *  For details, see <https://dun.github.io/munge/>.
+ *  For details, see <https://github.com/dun/munge>.
  *
  *  MUNGE is free software: you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
@@ -36,13 +35,6 @@
 #include "cipher.h"
 
 
-/*****************************************************************************
- *  Constants
- *****************************************************************************/
-
-#define CIPHER_MAGIC 0xDEADACE1
-
-
 /*****************************************************************************
  *  Private Data
  *****************************************************************************/
@@ -57,9 +49,9 @@ static int _cipher_is_initialized = 0;
 static void _cipher_init_subsystem (void);
 static int _cipher_init (cipher_ctx *x, munge_cipher_t cipher,
     unsigned char *key, unsigned char *iv, int enc);
-static int _cipher_update (cipher_ctx *x, void *dst, int *dstlen,
+static int _cipher_update (cipher_ctx *x, void *dst, int *dstlenp,
     const void *src, int srclen);
-static int _cipher_final (cipher_ctx *x, void *dst, int *dstlen);
+static int _cipher_final (cipher_ctx *x, void *dst, int *dstlenp);
 static int _cipher_cleanup (cipher_ctx *x);
 static int _cipher_block_size (munge_cipher_t cipher);
 static int _cipher_iv_size (munge_cipher_t cipher);
@@ -71,6 +63,9 @@ static int _cipher_map_enum (munge_cipher_t cipher, void *dst);
  *  Public Functions
  *****************************************************************************/
 
+/*  Initializes the cipher subsystem.
+ *  WARNING: This routine is *NOT* guaranteed to be thread-safe.
+ */
 void
 cipher_init_subsystem (void)
 {
@@ -84,6 +79,11 @@ cipher_init_subsystem (void)
 }
 
 
+/*  Initializes the cipher context [x] with cipher [cipher],
+ *    symmetric key [key], and initialization vector [iv].
+ *  The [enc] parm is set to 1 for encryption, and 0 for decryption.
+ *  Returns 0 on success, or -1 on error.
+ */
 int
 cipher_init (cipher_ctx *x, munge_cipher_t cipher,
              unsigned char *key, unsigned char *iv, int enc)
@@ -91,82 +91,85 @@ cipher_init (cipher_ctx *x, munge_cipher_t cipher,
     int rc;
 
     assert (_cipher_is_initialized);
-    assert (x != NULL);
-    assert (key != NULL);
-    assert (iv != NULL);
-    assert ((enc == 0) || (enc == 1));
 
-    rc = _cipher_init (x, cipher, key, iv, enc);
-    if (rc >= 0) {
-        assert (x->magic = CIPHER_MAGIC);
-        assert (!(x->finalized = 0));
+    if (!x || !key || !iv
+            || !((enc == CIPHER_DECRYPT) || (enc == CIPHER_ENCRYPT))) {
+        return (-1);
     }
+    rc = _cipher_init (x, cipher, key, iv, enc);
     return (rc);
 }
 
 
+/*  Updates the cipher context [x], reading [srclen] bytes from [src] and
+ *    writing the result into [dst] of length [dstlenp].  This can be called
+ *    multiple times to process successive blocks of data.
+ *  The number of bytes written will be from 0 to (srclen + cipher_block_size)
+ *    depending on the cipher block alignment.
+ *  Returns 0 on success, or -1 on error; in addition, [dstlenp] will be set
+ *    to the number of bytes written to [dst].
+ */
 int
-cipher_update (cipher_ctx *x, void *dst, int *dstlen,
+cipher_update (cipher_ctx *x, void *dst, int *dstlenp,
                const void *src, int srclen)
 {
     int rc;
 
     assert (_cipher_is_initialized);
-    assert (x != NULL);
-    assert (x->magic == CIPHER_MAGIC);
-    assert (x->finalized != 1);
-    assert (dst != NULL);
-    assert (dstlen != NULL);
-    assert (src != NULL);
 
-    if (srclen <= 0) {
-        return (0);
-    }
-    if ((dstlen == NULL) || (*dstlen <= 0)) {
+    if (!x || !dst || !dstlenp || (*dstlenp < 0) || !src || (srclen < 0)) {
         return (-1);
     }
-    rc = _cipher_update (x, dst, dstlen, src, srclen);
+    rc = _cipher_update (x, dst, dstlenp, src, srclen);
     return (rc);
 }
 
 
+/*  Finalizes the cipher context [x], processing the "final" data
+ *    remaining in a partial block and writing the result into [dst] of
+ *    length [dstlen].
+ *  The number of bytes written will be at most cipher_block_size() bytes
+ *    depending on the cipher block alignment.
+ *  After this function, no further calls to cipher_update() should be made.
+ *  Returns 0 on success, or -1 on error; in addition, [dstlenp] will be set
+ *    to the number of bytes written to [dst].
+ */
 int
-cipher_final (cipher_ctx *x, void *dst, int *dstlen)
+cipher_final (cipher_ctx *x, void *dst, int *dstlenp)
 {
     int rc;
 
     assert (_cipher_is_initialized);
-    assert (x != NULL);
-    assert (x->magic == CIPHER_MAGIC);
-    assert (x->finalized != 1);
-    assert (dst != NULL);
-    assert (dstlen != NULL);
 
-    if ((dstlen == NULL) || (*dstlen <= 0)) {
+    if (!x || !dst || !dstlenp || (*dstlenp < 0)) {
         return (-1);
     }
-    rc = _cipher_final (x, dst, dstlen);
-    assert (x->finalized = 1);
+    rc = _cipher_final (x, dst, dstlenp);
     return (rc);
 }
 
 
+/*  Clears the cipher context [x].
+ *  Returns 0 on success, or -1 on error.
+ */
 int
 cipher_cleanup (cipher_ctx *x)
 {
     int rc;
 
     assert (_cipher_is_initialized);
-    assert (x != NULL);
-    assert (x->magic == CIPHER_MAGIC);
 
+    if (!x) {
+        return (-1);
+    }
     rc = _cipher_cleanup (x);
     memset (x, 0, sizeof (*x));
-    assert (x->magic = ~CIPHER_MAGIC);
     return (rc);
 }
 
 
+/*  Returns the block size (in bytes) of the cipher [cipher], or -1 on error.
+ */
 int
 cipher_block_size (munge_cipher_t cipher)
 {
@@ -175,6 +178,9 @@ cipher_block_size (munge_cipher_t cipher)
 }
 
 
+/*  Returns the initialization vector length (in bytes) of the cipher [cipher],
+ *    0 if the cipher does not use an IV, or -1 on error.
+ */
 int
 cipher_iv_size (munge_cipher_t cipher)
 {
@@ -183,6 +189,8 @@ cipher_iv_size (munge_cipher_t cipher)
 }
 
 
+/*  Returns the key length (in bytes) of the cipher [cipher], or -1 on error.
+ */
 int
 cipher_key_size (munge_cipher_t cipher)
 {
@@ -191,6 +199,13 @@ cipher_key_size (munge_cipher_t cipher)
 }
 
 
+/*  Map the specified [cipher] algorithm to the internal representation used
+ *    by the underlying cryptographic library.
+ *  If [dst] is non-NULL, write the cryptographic library's internal
+ *    representation of the cipher algorithm to [dst]; otherwise, just validate
+ *    the specified [cipher] algorithm.
+ *  Returns 0 on success, or -1 on error.
+ */
 int
 cipher_map_enum (munge_cipher_t cipher, void *dst)
 {
@@ -212,7 +227,7 @@ cipher_map_enum (munge_cipher_t cipher, void *dst)
 
 static int _cipher_map [MUNGE_CIPHER_LAST_ITEM];
 
-static int _cipher_update_aux (cipher_ctx *x, void *dst, int *dstlen,
+static int _cipher_update_aux (cipher_ctx *x, void *dst, int *dstlenp,
     const void *src, int srclen);
 
 
@@ -283,13 +298,13 @@ _cipher_init (cipher_ctx *x, munge_cipher_t cipher,
 
 
 static int
-_cipher_update (cipher_ctx *x, void *vdst, int *dstlen,
+_cipher_update (cipher_ctx *x, void *vdst, int *dstlenp,
                 const void *vsrc, int srclen)
 {
 /*  During encryption, any remaining src data that is not a multiple of the
  *    cipher block size is saved in the context's partial block buffer.
  *    This buffer will be padded when the encryption is finalized
- *    (cf, PKCS #5, rfc2898).
+ *    (see PKCS #5, rfc2898).
  *  During decryption, the partial block buffer will always contain data at
  *    the end of each update to ensure the padding is properly removed when
  *    the decryption is finalized.
@@ -320,7 +335,7 @@ _cipher_update (cipher_ctx *x, void *vdst, int *dstlen,
          */
         if (x->len == x->blklen) {
             if ((x->do_encrypt) || (srclen > 0)) {
-                n = *dstlen;
+                n = *dstlenp;
                 if (_cipher_update_aux (x, dst, &n, x->buf, x->blklen) < 0) {
                     goto err;
                 }
@@ -347,7 +362,7 @@ _cipher_update (cipher_ctx *x, void *vdst, int *dstlen,
     if (n_complete > 0) {
         assert (x->len == 0);
         assert (n_complete % x->blklen == 0);
-        n = *dstlen - n_written;
+        n = *dstlenp - n_written;
         if (_cipher_update_aux (x, dst, &n, src, n_complete) < 0) {
             goto err;
         }
@@ -370,21 +385,21 @@ _cipher_update (cipher_ctx *x, void *vdst, int *dstlen,
 
     /*  Set the number of bytes written.
      */
-    *dstlen = n_written;
+    *dstlenp = n_written;
     return (0);
 
 err:
-    *dstlen = 0;
+    *dstlenp = 0;
     return (-1);
 }
 
 
 static int
-_cipher_update_aux (cipher_ctx *x, void *dst, int *dstlen_ptr,
+_cipher_update_aux (cipher_ctx *x, void *dst, int *dstlenp,
                     const void *src, int srclen)
 {
     gcry_error_t e;
-    int          dstlen = *dstlen_ptr;
+    int          dstlen = *dstlenp;
 
     if (x->do_encrypt) {
         e = gcry_cipher_encrypt (x->ctx, dst, dstlen, src, srclen);
@@ -396,18 +411,18 @@ _cipher_update_aux (cipher_ctx *x, void *dst, int *dstlen_ptr,
         log_msg (LOG_DEBUG, "%s failed: %s",
             (x->do_encrypt ? "gcry_cipher_encrypt" : "gcry_cipher_decrypt"),
             gcry_strerror (e));
-        *dstlen_ptr = 0;
+        *dstlenp = 0;
         return (-1);
     }
     if ((src != NULL) || (srclen != 0)) {
-        *dstlen_ptr = srclen;
+        *dstlenp = srclen;
     }
     return (0);
 }
 
 
 static int
-_cipher_final (cipher_ctx *x, void *dst, int *dstlen)
+_cipher_final (cipher_ctx *x, void *dst, int *dstlenp)
 {
     int n;
     int i;
@@ -419,7 +434,7 @@ _cipher_final (cipher_ctx *x, void *dst, int *dstlen)
         for (i = x->len; i < x->blklen; i++) {
             x->buf[i] = pad;
         }
-        if (_cipher_update_aux (x, dst, dstlen, x->buf, x->blklen) < 0) {
+        if (_cipher_update_aux (x, dst, dstlenp, x->buf, x->blklen) < 0) {
             return (-1);
         }
     }
@@ -459,12 +474,12 @@ _cipher_final (cipher_ctx *x, void *dst, int *dstlen)
          */
         n = x->blklen - pad;
         if (n > 0) {
-            if (*dstlen < n) {
+            if (*dstlenp < n) {
                 return (-1);
             }
             memcpy (dst, x->buf, n);
         }
-        *dstlen = n;
+        *dstlenp = n;
     }
     return (0);
 }
@@ -550,23 +565,10 @@ _cipher_map_enum (munge_cipher_t cipher, void *dst)
 /*****************************************************************************
  *  Private Functions (OpenSSL)
  *****************************************************************************/
-/*
- *  EVP_CipherInit() implicitly initializes the EVP_CIPHER_CTX.
- *    This call has been deprecated as of OpenSSL 0.9.7.
- *  EVP_CipherInit(), EVP_CipherUpdate(), and EVP_CIPHER_CTX_cleanup()
- *    return void in OpenSSL 0.9.5a and earlier versions, and int in later
- *    versions.  I'm using EVP_CipherInit_ex() as my test for this behavior.
- *    This probably isn't the best test since it fails for OpenSSL 0.9.6b.
- *    But this isn't as bad as it sounds since software versions of these
- *    functions will never return errors (unless there is a programming error),
- *    and hardware versions require the EVP_CipherInit_ex() interface provided
- *    by OpenSSL 0.9.7.
- *  If EVP_CipherInit_ex() exists, so should EVP_CIPHER_CTX_init().
- *    But EVP_CIPHER_CTX_cleanup() exists in the versions of which I'm aware.
- */
 
 #if HAVE_OPENSSL
 
+#include <openssl/crypto.h>
 #include <openssl/evp.h>
 
 static const EVP_CIPHER *_cipher_map [MUNGE_CIPHER_LAST_ITEM];
@@ -604,45 +606,79 @@ _cipher_init (cipher_ctx *x, munge_cipher_t cipher,
     if (_cipher_map_enum (cipher, &algo) < 0) {
         return (-1);
     }
-#if HAVE_EVP_CIPHERINIT_EX
-    EVP_CIPHER_CTX_init (&(x->ctx));
-    if (!(EVP_CipherInit_ex (&(x->ctx), algo, NULL, key, iv, enc))) {
+#if HAVE_EVP_CIPHER_CTX_NEW
+    /*  OpenSSL >= 0.9.8b  */
+    x->ctx = EVP_CIPHER_CTX_new ();
+#else  /* !HAVE_EVP_CIPHER_CTX_NEW */
+    x->ctx = OPENSSL_malloc (sizeof (EVP_CIPHER_CTX));
+#endif /* !HAVE_EVP_CIPHER_CTX_NEW */
+    if (x->ctx == NULL) {
         return (-1);
     }
-#else  /* !HAVE_EVP_CIPHERINIT_EX */
-    EVP_CipherInit (&(x->ctx), algo, key, iv, enc);
-#endif /* !HAVE_EVP_CIPHERINIT_EX */
+
+#if HAVE_EVP_CIPHERINIT_EX
+#if HAVE_EVP_CIPHER_CTX_INIT
+    /*  OpenSSL >= 0.9.7, < 1.1.0  */
+    EVP_CIPHER_CTX_init (x->ctx);
+#endif /* HAVE_EVP_CIPHER_CTX_INIT */
+    /*  OpenSSL >= 0.9.7  */
+    if (EVP_CipherInit_ex (x->ctx, algo, NULL, key, iv, enc) != 1) {
+        return (-1);
+    }
+#elif HAVE_EVP_CIPHERINIT_RETURN_INT
+    /*  EVP_CipherInit() implicitly initializes the EVP_CIPHER_CTX.  */
+    /*  OpenSSL > 0.9.5a  */
+    if (EVP_CipherInit (x->ctx, algo, key, iv, enc) != 1) {
+        return (-1);
+    }
+#elif HAVE_EVP_CIPHERINIT
+    /*  EVP_CipherInit() implicitly initializes the EVP_CIPHER_CTX.  */
+    /*  OpenSSL <= 0.9.5a  */
+    EVP_CipherInit (x->ctx, algo, key, iv, enc);
+#else  /* !HAVE_EVP_CIPHERINIT */
+#error "No OpenSSL EVP_CipherInit"
+#endif /* !HAVE_EVP_CIPHERINIT */
+
     return (0);
 }
 
 
 static int
-_cipher_update (cipher_ctx *x, void *dst, int *dstlen,
+_cipher_update (cipher_ctx *x, void *dst, int *dstlenp,
                 const void *src, int srclen)
 {
-#if HAVE_EVP_CIPHERINIT_EX
-    if (!(EVP_CipherUpdate (&(x->ctx), dst, dstlen, (void *) src, srclen))) {
+#if HAVE_EVP_CIPHERUPDATE_RETURN_INT
+    /*  OpenSSL > 0.9.5a  */
+    if (EVP_CipherUpdate (x->ctx, dst, dstlenp, (void *) src, srclen) != 1) {
         return (-1);
     }
-#else  /* !HAVE_EVP_CIPHERINIT_EX */
-    EVP_CipherUpdate (&(x->ctx), dst, dstlen, (void *) src, srclen);
-#endif /* !HAVE_EVP_CIPHERINIT_EX */
+#elif HAVE_EVP_CIPHERUPDATE
+    /*  OpenSSL <= 0.9.5a  */
+    EVP_CipherUpdate (x->ctx, dst, dstlenp, (void *) src, srclen);
+#else  /* !HAVE_EVP_CIPHERUPDATE */
+#error "No OpenSSL EVP_CipherUpdate"
+#endif /* !HAVE_EVP_CIPHERUPDATE */
+
     return (0);
 }
 
 
 static int
-_cipher_final (cipher_ctx *x, void *dst, int *dstlen)
+_cipher_final (cipher_ctx *x, void *dst, int *dstlenp)
 {
-#if HAVE_EVP_CIPHERINIT_EX
-    if (!(EVP_CipherFinal_ex (&(x->ctx), dst, dstlen))) {
+#if HAVE_EVP_CIPHERFINAL_EX
+    /*  OpenSSL >= 0.9.7  */
+    if (EVP_CipherFinal_ex (x->ctx, dst, dstlenp) != 1) {
         return (-1);
     }
-#else  /* !HAVE_EVP_CIPHERINIT_EX */
-    if (!(EVP_CipherFinal (&(x->ctx), dst, dstlen))) {
+#elif HAVE_EVP_CIPHERFINAL
+    if (EVP_CipherFinal (x->ctx, dst, dstlenp) != 1) {
         return (-1);
     }
-#endif /* !HAVE_EVP_CIPHERINIT_EX */
+#else  /* !HAVE_EVP_CIPHERFINAL */
+#error "No OpenSSL EVP_CipherFinal"
+#endif /* !HAVE_EVP_CIPHERFINAL */
+
     return (0);
 }
 
@@ -650,14 +686,26 @@ _cipher_final (cipher_ctx *x, void *dst, int *dstlen)
 static int
 _cipher_cleanup (cipher_ctx *x)
 {
-#if HAVE_EVP_CIPHERINIT_EX
-    if (!(EVP_CIPHER_CTX_cleanup (&(x->ctx)))) {
-        return (-1);
+    int rv = 0;
+
+#if HAVE_EVP_CIPHER_CTX_FREE
+    /*  OpenSSL >= 0.9.8b  */
+    EVP_CIPHER_CTX_free (x->ctx);
+#else  /* !HAVE_EVP_CIPHER_CTX_FREE */
+#if HAVE_EVP_CIPHER_CTX_CLEANUP_RETURN_INT
+    /*  OpenSSL > 0.9.5a, < 1.1.0  */
+    if (EVP_CIPHER_CTX_cleanup (x->ctx) != 1) {
+        rv = -1;
     }
-#else  /* !HAVE_EVP_CIPHERINIT_EX */
-    EVP_CIPHER_CTX_cleanup (&(x->ctx));
-#endif /* !HAVE_EVP_CIPHERINIT_EX */
-    return (0);
+#elif HAVE_EVP_CIPHER_CTX_CLEANUP
+    /*  OpenSSL <= 0.9.5a  */
+    EVP_CIPHER_CTX_cleanup (x->ctx);
+#endif /* HAVE_EVP_CIPHER_CTX_CLEANUP */
+    OPENSSL_free (x->ctx);
+#endif /* !HAVE_EVP_CIPHER_CTX_FREE */
+
+    x->ctx = NULL;
+    return (rv);
 }
 
 
