@@ -1,11 +1,10 @@
 /*****************************************************************************
- *  Written by Chris Dunlap <cdunlap@llnl.gov>.
- *  Copyright (C) 2007-2016 Lawrence Livermore National Security, LLC.
+ *  Copyright (C) 2007-2025 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2002-2007 The Regents of the University of California.
  *  UCRL-CODE-155910.
  *
  *  This file is part of the MUNGE Uid 'N' Gid Emporium (MUNGE).
- *  For details, see <https://dun.github.io/munge/>.
+ *  For details, see <https://github.com/dun/munge>.
  *
  *  MUNGE is free software: you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
@@ -31,10 +30,10 @@
 #endif /* HAVE_CONFIG_H */
 
 #include <assert.h>
-#include <sys/types.h>                  /* include before in.h for bsd */
 #include <netinet/in.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/types.h>
 #include <time.h>
 #include "auth_recv.h"
 #include "base64.h"
@@ -183,10 +182,15 @@ enc_validate_msg (m_msg_t m)
     if (m->data_len == 0) {
         m->zip = MUNGE_ZIP_NONE;
     }
+    else if (m->data_len > MUNGE_MAXIMUM_PAYLOAD_LEN) {
+        return (m_msg_set_err (m, EMUNGE_BAD_LENGTH,
+            strdupf ("Payload size %lu exceeded maximum of %lu",
+                m->data_len, MUNGE_MAXIMUM_PAYLOAD_LEN)));
+    }
     /*  Validate realm.
      *
      *  FIXME: Validate realm and set default string if needed.
-     *         Validate that the realm string is NUL-terminated.
+     *         Validate that the realm string is null-terminated.
      */
     /*  Validate time-to-live.
      *  Ensure it is bounded by the configuration's max ttl.
@@ -267,12 +271,12 @@ enc_check_retry (munge_cred_t c)
 
     if (m->retry > 0) {
         log_msg (LOG_INFO,
-            "Encode retry #%d for client UID=%d GID=%d",
-            m->retry, m->client_uid, m->client_gid);
+            "Encode retry #%d for client UID=%u GID=%u", m->retry,
+            (unsigned int) m->client_uid, (unsigned int) m->client_gid);
     }
     if (m->retry > MUNGE_SOCKET_RETRY_ATTEMPTS) {
         return (m_msg_set_err (m, EMUNGE_SOCKET,
-            strdupf ("Exceeded maximum number of encode attempts")));
+            strdup ("Exceeded maximum number of encode attempts")));
     }
     return (0);
 }
@@ -676,10 +680,12 @@ enc_armor (munge_cred_t c)
     unsigned char *buf;                 /* armor'd data buffer               */
     unsigned char *buf_ptr;             /* ptr into armor'd data buffer      */
     base64_ctx     x;                   /* base64 context                    */
-    int            n, n2;               /* all-purpose ints                  */
+    int            n;                   /* temporary int                     */
 
-    prefix_len = strlen (MUNGE_CRED_PREFIX);
-    suffix_len = strlen (MUNGE_CRED_SUFFIX);
+    prefix_len = sizeof MUNGE_CRED_PREFIX - 1;
+    assert (prefix_len > 0);
+    suffix_len = sizeof MUNGE_CRED_SUFFIX - 1;
+    assert (suffix_len > 0);
 
     /*  Allocate memory for armor'd data.
      */
@@ -691,50 +697,39 @@ enc_armor (munge_cred_t c)
     }
     buf_ptr = buf;
 
-    /*  Add the prefix string.
+    /*  Add the prefix string w/o null termination since more data will follow.
      */
-    if (prefix_len > 0) {
-        strcpy ((char *) buf_ptr, MUNGE_CRED_PREFIX); /* strcpy() safe here */
-        buf_ptr += prefix_len;
-    }
+    memcpy (buf_ptr, MUNGE_CRED_PREFIX, prefix_len);
+    buf_ptr += prefix_len;
+
     /*  Base64-encode the chewy-internals of the credential.
-     *  The data will be NUL-terminated by in the process.
      */
     if (base64_init (&x) < 0) {
         goto err;
     }
-    n = 0;
-    if (base64_encode_update (&x, buf_ptr, &n2, c->outer, c->outer_len) < 0) {
+    if (base64_encode_update (&x, buf_ptr, &n, c->outer, c->outer_len) < 0) {
         goto err_cleanup;
     }
-    buf_ptr += n2;
-    n += n2;
-    if (base64_encode_update (&x, buf_ptr, &n2, c->mac, c->mac_len) < 0) {
+    buf_ptr += n;
+    if (base64_encode_update (&x, buf_ptr, &n, c->mac, c->mac_len) < 0) {
         goto err_cleanup;
     }
-    buf_ptr += n2;
-    n += n2;
-    if (base64_encode_update (&x, buf_ptr, &n2, c->inner, c->inner_len) < 0) {
+    buf_ptr += n;
+    if (base64_encode_update (&x, buf_ptr, &n, c->inner, c->inner_len) < 0) {
         goto err_cleanup;
     }
-    buf_ptr += n2;
-    n += n2;
-    if (base64_encode_final (&x, buf_ptr, &n2) < 0) {
+    buf_ptr += n;
+    if (base64_encode_final (&x, buf_ptr, &n) < 0) {
         goto err_cleanup;
     }
-    buf_ptr += n2;
-    n += n2;
+    buf_ptr += n;
     if (base64_cleanup (&x) < 0) {
         goto err;
     }
-    n++;                                /* count the terminating NUL char */
-
-    /*  Add the suffix string.
+    /*  Add the suffix string with null termination since this is the end.
      */
-    if (suffix_len > 0) {
-        strcpy ((char *) buf_ptr, MUNGE_CRED_SUFFIX); /* strcpy() safe here */
-        buf_ptr += suffix_len;
-    }
+    memcpy (buf_ptr, MUNGE_CRED_SUFFIX, suffix_len + 1);
+    buf_ptr += suffix_len;
     assert ((buf_ptr - buf) < buf_len);
 
     /*  Replace "outer+inner" data with armor'd data.
