@@ -1,11 +1,10 @@
 /*****************************************************************************
- *  Written by Chris Dunlap <cdunlap@llnl.gov>.
- *  Copyright (C) 2007-2022 Lawrence Livermore National Security, LLC.
+ *  Copyright (C) 2007-2025 Lawrence Livermore National Security, LLC.
  *  Copyright (C) 2002-2007 The Regents of the University of California.
  *  UCRL-CODE-155910.
  *
  *  This file is part of the MUNGE Uid 'N' Gid Emporium (MUNGE).
- *  For details, see <https://dun.github.io/munge/>.
+ *  For details, see <https://github.com/dun/munge>.
  *
  *  MUNGE is free software: you can redistribute it and/or modify it under
  *  the terms of the GNU General Public License as published by the Free
@@ -81,6 +80,11 @@ static void * _work_dequeue (work_p wp);
  *  Public Functions
  *****************************************************************************/
 
+/*  Initializes the work crew comprised of [n_threads] workers.
+ *    The work function [f] will be invoked to process each work element
+ *    queued by work_queue().
+ *  Returns a ptr to the work crew, or terminates on error.
+ */
 work_p
 work_init (work_func_t f, int n_threads)
 {
@@ -89,16 +93,9 @@ work_init (work_func_t f, int n_threads)
     size_t stacksize = 256 * 1024;
     int i;
 
-    /*  Check args.
-     */
-    if (f == NULL) {
-        errno = EINVAL;
-        return (NULL);
-    }
-    if (n_threads <= 0) {
-        errno = EINVAL;
-        return (NULL);
-    }
+    assert (f != NULL);
+    assert (n_threads > 0);
+
     /*  Allocate memory.
      */
     if (!(wp = malloc (sizeof (work_t)))) {
@@ -160,10 +157,17 @@ work_init (work_func_t f, int n_threads)
         log_errno (EMUNGE_SNAFU, LOG_ERR,
             "Failed to destroy work thread attribute");
     }
+    log_msg (LOG_INFO, "Created %d work thread%s", n_threads,
+            ((n_threads > 1) ? "s" : ""));
     return (wp);
 }
 
 
+/*  Stops the work crew [wp], canceling all worker threads and releasing
+ *    associated resources.  If [do_wait] is non-zero, all currently-queued
+ *    work will be processed before the work crew is stopped; new work is
+ *    prevented from being added to the queue during this time.
+ */
 void
 work_fini (work_p wp, int do_wait)
 {
@@ -246,6 +250,10 @@ work_fini (work_p wp, int do_wait)
 }
 
 
+/*  Queues the [work] element for processing by the work crew [wp].
+ *    The [work] will be passed to the function specified during work_init().
+ *  Returns 0 on success, or -1 on error (with errno set).
+ */
 int
 work_queue (work_p wp, void *work)
 {
@@ -289,6 +297,8 @@ work_queue (work_p wp, void *work)
 }
 
 
+/*  Waits until all queued work is processed by the work crew [wp].
+ */
 void
 work_wait (work_p wp)
 {
